==> gmssl-sdf-crypto-0.1-sources.jar: com/xdja/pki/gmssl/crypto/sdf/SdfSM2Signer.java <==
package com.xdja.pki.gmssl.crypto.sdf;

import com.xdja.pki.gmssl.core.utils.GMSSLByteArrayUtils;
import com.xdja.pki.gmssl.sdf.SdfSDK;
import com.xdja.pki.gmssl.sdf.SdfSDKException;
import com.xdja.pki.gmssl.sdf.bean.SdfECCPublicKey;
import com.xdja.pki.gmssl.sdf.bean.SdfECCSignature;
import org.bouncycastle.asn1.*;
import org.bouncycastle.crypto.CipherParameters;
import org.bouncycastle.crypto.CryptoException;
import org.bouncycastle.crypto.DataLengthException;
import org.bouncycastle.crypto.Signer;
import org.bouncycastle.math.ec.ECConstants;
import org.bouncycastle.util.Arrays;
import org.bouncycastle.util.BigIntegers;
import org.bouncycastle.util.encoders.Hex;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;
import java.math.BigInteger;

public class SdfSM2Signer implements Signer, ECConstants {

    private Logger logger = LoggerFactory.getLogger(this.getClass());

    private SdfSDK sdfSDK;

    private SdfSM3Digest digest;

    private SdfECKeyParameters ecKey;

    public SdfSM2Signer() throws SdfSDKException {
        this(SdfCryptoType.YUNHSUM);
    }

    public SdfSM2Signer(SdfCryptoType sdfCryptoType) throws SdfSDKException {
        this(sdfCryptoType.getSdfSDK());
    }

    public SdfSM2Signer(SdfSDK sdfSDK) throws SdfSDKException {
        this.sdfSDK = sdfSDK;
        this.sdfSDK.init();
    }

    @Override
    public void init(boolean forSigning, CipherParameters param) {
        this.ecKey = (SdfECKeyParameters) param;
        byte[] userID = Hex.decode("31323334353637383132333435363738"); // the default value
        try {
            SdfECCPublicKey sdfECCPublicKey;
            if (forSigning){
                sdfECCPublicKey = this.sdfSDK.exportSignPublicKeyEcc(this.ecKey.getSm2Index());
            } else {
                sdfECCPublicKey = this.ecKey.getSDFECCPublicKey();
            }
            digest = new SdfSM3Digest(sdfSDK, userID, sdfECCPublicKey);
        } catch (SdfSDKException e) {
            logger.error("init", e);
        }
    }

    @Override
    public void update(byte b) {
        byte[] bytes = new byte[]{b};
        update(bytes, 0, bytes.length);
    }

    @Override
    public void update(byte[] in, int off, int len) {
        digest.update(in, off, len);
    }

    @Override
    public byte[] generateSignature() throws CryptoException, DataLengthException {
        byte[] out = new byte[digest.getDigestSize()];
        digest.doFinal(out, 0);
        GMSSLByteArrayUtils.printHexBinary(logger, "signature sm3 digest", out);
        try {
            //获取私钥访问权限
            sdfSDK.getPrivateKeyAccessRight(ecKey.getSm2Index(), ecKey.getPassword());
            //SM2签名
            SdfECCSignature sdfECCSignature = sdfSDK.internalSignECC(ecKey.getSm2Index(), out);
            //释放私钥访问权限
            sdfSDK.releasePrivateKeyAccessRight(ecKey.getSm2Index());
            return derEncode(sdfECCSignature.getR(), sdfECCSignature.getS());
        } catch (SdfSDKException | IOException e) {
            logger.error("generateSignature", e);
            return new byte[0];
        }
    }

    @Override
    public boolean verifySignature(byte[] signature) {
        try {
            byte[] out = new byte[digest.getDigestSize()];
            digest.doFinal(out, 0);
            GMSSLByteArrayUtils.printHexBinary(logger, "signature sm3 digest", out);
            BigInteger[] bigIntegers = derDecode(signature);
            byte[] r = GMSSLByteArrayUtils.filterByteArrayZeroInHead(bigIntegers[0].toByteArray());
            GMSSLByteArrayUtils.printHexBinary(logger, "verifySignature R", r);
            byte[] s = GMSSLByteArrayUtils.filterByteArrayZeroInHead(bigIntegers[1].toByteArray());
            GMSSLByteArrayUtils.printHexBinary(logger, "verifySignature S", s);
            SdfECCSignature sdfECCSignature = new SdfECCSignature(r, s);
            //SM2验签
            sdfSDK.externalVerifyECC(ecKey.getSDFECCPublicKey(), out, sdfECCSignature);
            return true;
        } catch (IOException | SdfSDKException e) {
            logger.error("verifySignature", e);
            return false;
        }
    }

    @Override
    public void reset() {

    }

    protected BigInteger[] derDecode(byte[] encoding) throws IOException {
        ASN1Sequence seq = ASN1Sequence.getInstance(ASN1Primitive.fromByteArray(encoding));
        if (seq.size() != 2) {
            return null;
        }

        BigInteger r = ASN1Integer.getInstance(seq.getObjectAt(0)).getValue();
        BigInteger s = ASN1Integer.getInstance(seq.getObjectAt(1)).getValue();

        byte[] expectedEncoding = derEncode(r, s);
        if (!Arrays.constantTimeAreEqual(expectedEncoding, encoding)) {
            return null;
        }

        return new BigInteger[]{r, s};
    }

    protected byte[] derEncode(byte[] r, byte[] s) throws IOException {
        return derEncode(
                BigIntegers.fromUnsignedByteArray(r),
                BigIntegers.fromUnsignedByteArray(s)
        );
    }

    protected byte[] derEncode(BigInteger r, BigInteger s) throws IOException {
        GMSSLByteArrayUtils.printHexBinary(logger, "r", r.toByteArray());
        GMSSLByteArrayUtils.printHexBinary(logger, "s", s.toByteArray());
        ASN1EncodableVector v = new ASN1EncodableVector();
        v.add(new ASN1Integer(r));
        v.add(new ASN1Integer(s));
        return new DERSequence(v).getEncoded(ASN1Encoding.DER);
    }

    public void release() throws SdfSDKException {
        if (sdfSDK != null) {
            sdfSDK.release();
        }
    }
}

==> gmssl-sdf-crypto-0.1.1-20190627.072250-3-sources.jar: com/xdja/pki/gmssl/crypto/sdf/SdfSM2Signer.java <==
package com.xdja.pki.gmssl.crypto.sdf;

import com.xdja.pki.gmssl.core.utils.GMSSLByteArrayUtils;
import com.xdja.pki.gmssl.sdf.SdfSDK;
import com.xdja.pki.gmssl.sdf.SdfSDKException;
import com.xdja.pki.gmssl.sdf.bean.SdfECCPublicKey;
import com.xdja.pki.gmssl.sdf.bean.SdfECCSignature;
import org.bouncycastle.asn1.*;
import org.bouncycastle.crypto.CipherParameters;
import org.bouncycastle.crypto.CryptoException;
import org.bouncycastle.crypto.DataLengthException;
import org.bouncycastle.crypto.Signer;
import org.bouncycastle.math.ec.ECConstants;
import org.bouncycastle.util.Arrays;
import org.bouncycastle.util.BigIntegers;
import org.bouncycastle.util.encoders.Hex;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;
import java.math.BigInteger;

public class SdfSM2Signer implements Signer, ECConstants {

    private Logger logger = LoggerFactory.getLogger(this.getClass());

    private SdfSDK sdfSDK;

    private SdfSM3Digest digest;

    private SdfECKeyParameters ecKey;

    public SdfSM2Signer() throws SdfSDKException {
        this(SdfCryptoType.YUNHSM);
    }

    public SdfSM2Signer(SdfCryptoType sdfCryptoType) throws SdfSDKException {
        this(sdfCryptoType.getSdfSDK());
    }

    public SdfSM2Signer(SdfSDK sdfSDK) throws SdfSDKException {
        this.sdfSDK = sdfSDK;
        this.sdfSDK.init();
    }

    @Override
    public void init(boolean forSigning, CipherParameters param) {
        this.ecKey = (SdfECKeyParameters) param;
        byte[] userID = GMSSLByteArrayUtils.hexDecode("31323334353637383132333435363738"); // the default value
        try {
            SdfECCPublicKey sdfECCPublicKey;
            if (forSigning) {
                sdfECCPublicKey = this.sdfSDK.exportSignPublicKeyEcc(this.ecKey.getSm2Index());
            } else {
                sdfECCPublicKey = this.ecKey.getSDFECCPublicKey();
            }
            digest = new SdfSM3Digest(sdfSDK, userID, sdfECCPublicKey);
        } catch (SdfSDKException e) {
            logger.error("init", e);
        }
    }

    @Override
    public void update(byte b) {
        byte[] bytes = new byte[]{b};
        update(bytes, 0, bytes.length);
    }

    @Override
    public void update(byte[] in, int off, int len) {
        while (len > 4096) {
            GMSSLByteArrayUtils.printBase64Binary(logger, "in", in);
            byte[] m = new byte[4096];
            System.arraycopy(in, 0, m, 0, m.length);
            digest.update(m, 0, m.length);
            len = len - 4096;
            byte[] inNow = new byte[in.length - m.length];
            logger.debug("m.length " + m.length);
            logger.debug("in.length " +in.length);
            logger.debug("inNow.length " +inNow.length);
            System.arraycopy(in,  m.length, inNow, 0, in.length - m.length);
            in = inNow;
            GMSSLByteArrayUtils.printBase64Binary(logger, "in", inNow);
        }
        digest.update(in, off, len);
    }

    @Override
    public byte[] generateSignature() throws CryptoException, DataLengthException {
        byte[] out = new byte[digest.getDigestSize()];
        digest.doFinal(out, 0);
        GMSSLByteArrayUtils.printHexBinary(logger, "signature sm3 digest", out);
        try {
            //获取私钥访问权限
            sdfSDK.getPrivateKeyAccessRight(ecKey.getSm2Index(), ecKey.getPassword());
            //SM2签名
            SdfECCSignature sdfECCSignature = sdfSDK.internalSignECC(ecKey.getSm2Index(), ecKey.getPassword(), out);
            //释放私钥访问权限
            sdfSDK.releasePrivateKeyAccessRight(ecKey.getSm2Index());
            return derEncode(sdfECCSignature.getR(), sdfECCSignature.getS());
        } catch (SdfSDKException | IOException e) {
            logger.error("generateSignature", e);
            return new byte[0];
        }
    }

    @Override
    public boolean verifySignature(byte[] signature) {
        try {
            byte[] out = new byte[digest.getDigestSize()];
            digest.doFinal(out, 0);
            GMSSLByteArrayUtils.printHexBinary(logger, "signature sm3 digest", out);
            BigInteger[] bigIntegers = derDecode(signature);
            byte[] r = GMSSLByteArrayUtils.filterByteArrayZeroInHead(bigIntegers[0].toByteArray());
            GMSSLByteArrayUtils.printHexBinary(logger, "verifySignature R", r);
            byte[] s = GMSSLByteArrayUtils.filterByteArrayZeroInHead(bigIntegers[1].toByteArray());
            GMSSLByteArrayUtils.printHexBinary(logger, "verifySignature S", s);
            SdfECCSignature sdfECCSignature = new SdfECCSignature(r, s);
            //SM2验签
            sdfSDK.externalVerifyECC(ecKey.getSDFECCPublicKey(), out, sdfECCSignature);
            return true;
        } catch (IOException | SdfSDKException e) {
            logger.error("verifySignature", e);
            return false;
        }
    }

    @Override
    public void reset() {

    }

    protected BigInteger[] derDecode(byte[] encoding) throws IOException {
        ASN1Sequence seq = ASN1Sequence.getInstance(ASN1Primitive.fromByteArray(encoding));
        if (seq.size() != 2) {
            return null;
        }

        BigInteger r = ASN1Integer.getInstance(seq.getObjectAt(0)).getValue();
        BigInteger s = ASN1Integer.getInstance(seq.getObjectAt(1)).getValue();

        byte[] expectedEncoding = derEncode(r, s);
        if (!Arrays.constantTimeAreEqual(expectedEncoding, encoding)) {
            return null;
        }

        return new BigInteger[]{r, s};
    }

    protected byte[] derEncode(byte[] r, byte[] s) throws IOException {
        return derEncode(
                BigIntegers.fromUnsignedByteArray(r),
                BigIntegers.fromUnsignedByteArray(s)
        );
    }

    protected byte[] derEncode(BigInteger r, BigInteger s) throws IOException {
        GMSSLByteArrayUtils.printHexBinary(logger, "r", r.toByteArray());
        GMSSLByteArrayUtils.printHexBinary(logger, "s", s.toByteArray());
        ASN1EncodableVector v = new ASN1EncodableVector();
        v.add(new ASN1Integer(r));
        v.add(new ASN1Integer(s));
        return new DERSequence(v).getEncoded(ASN1Encoding.DER);
    }

    public void release() throws SdfSDKException {
        if (sdfSDK != null) {
            sdfSDK.release();
        }
    }
}
